@@ -19,20 +19,21 @@ from torch.utils.data import DataLoader
 
 from anndata import AnnData
 import scanpy as sc
-from sklearn.utils.sparsefuncs import inplace_row_scale, inplace_column_scale
-from multiprocessing import Pool, cpu_count
+from sklearn.preprocessing import maxabs_scale, MaxAbsScaler
 
 from glob import glob
-import psutil # TEST
 
 np.warnings.filterwarnings('ignore')
 DATA_PATH = os.path.expanduser("~")+'/.scalex/'
-CHUNK_SIZE = 100000
+CHUNK_SIZE = 20000
 
 
 def read_mtx(path):
     """
-    Read mtx format data folder
+    Read mtx format data folder including: 
+        matrix file: e.g. count.mtx or matrix.mtx
+        barcode file: e.g. barcode.txt
+        feature file: e.g. feature.txt
     """
     for filename in glob(path+'/*'):
         if ('count' in filename or 'matrix' in filename or 'data' in filename) and ('mtx' in filename):
@@ -51,6 +52,9 @@ def read_mtx(path):
 
 
 def load_file(path):  
+    """
+    Load single cell dataset from file
+    """
     if os.path.exists(DATA_PATH+path+'.h5ad'):
         adata = sc.read_h5ad(DATA_PATH+path+'.h5ad')
     elif os.path.isdir(path): # mtx format
@@ -74,7 +78,7 @@ def load_file(path):
 
 def load_files(root):
     """
-    Load single cell dataset based-on the path
+    Load single cell dataset from files
     """
     if root.split('/')[-1] == '*':
         adata = []
@@ -117,34 +121,62 @@ def concat_data(
     return concat
         
     
-def preprocessing(adata, 
+def preprocessing(
+        adata, 
         min_genes=600, 
         min_cells=3, 
         target_sum=1e4, 
         n_top_genes=2000, # or gene list
         chunk_size=CHUNK_SIZE,
+        log=None
     ):
     """
     preprocessing
     """
-    print('Preprocessing')
+    if log: log.info('Preprocessing')
     if type(adata.X) == np.ndarray:
         adata.X = scipy.sparse.csr_matrix(adata.X)
     
     adata = adata[:, [gene for gene in adata.var_names 
                   if not str(gene).startswith(tuple(['ERCC', 'MT-', 'mt-']))]]
     
-    sc.pp.filter_cells(adata, min_genes=min_genes)
+    if log: log.info('Filtering cells')
+    cell_subsets = []
+    for i in range(adata.shape[0]//chunk_size+1):
+        cell_subset, _ = sc.pp.filter_cells(adata[i*chunk_size:(i+1)*chunk_size], min_genes=min_genes, inplace=False)
+        cell_subsets.append(cell_subset)
+    indices = np.where(np.concatenate(cell_subsets))[0]
+    adata = adata[indices]
+
+#     sc.pp.filter_cells(adata, min_genes=min_genes)
+    
+    if log: log.info('Filtering genes')
+#     adata = adata[np.where(np.sum(adata.X>0, 0))]
     sc.pp.filter_genes(adata, min_cells=min_cells)
-    for chunk, start, end in adata.chunked_X(chunk_size):
-        adata.X[start:end] = sc.pp.normalize_total(adata[start:end], target_sum=target_sum, exclude_highly_expressed=True, inplace=False)['X']
-#     sc.pp.normalize_total(adata, target_sum=target_sum, exclude_highly_expressed=True)
-    sc.pp.log1p(adata, chunked=True, chunk_size=chunk_size)
+    from sklearn.utils import sparsefuncs
+    if log: log.info('Normalizing total to {}'.format(target_sum))
+#     for i in range(adata.shape[0]//chunk_size+1):
+#         log.info(i)
+#         counts = np.ravel(adata.X[i*chunk_size:(i+1)*chunk_size].sum(1))
+#         log.info('counts')
+#         sparsefuncs.inplace_row_scale(adata.X[i*chunk_size:(i+1)*chunk_size], target_sum/counts)
+#         adata.X[i*chunk_size:(i+1)*chunk_size] = sc.pp.normalize_total(adata[i*chunk_size:(i+1)*chunk_size], target_sum=target_sum, exclude_highly_expressed=True, inplace=False)['X']
+    sc.pp.normalize_total(adata, target_sum=target_sum, exclude_highly_expressed=True)
+
+    if log: log.info('Log1p transforming')
+    sc.pp.log1p(adata) #, chunked=True, chunk_size=chunk_size)
+    
+    if log: log.info('Finding variable genes')
     if type(n_top_genes) == int and n_top_genes>0:
         sc.pp.highly_variable_genes(adata, n_top_genes=n_top_genes, batch_key='batch', inplace=False, subset=True)
     else:
         adata = reindex(adata, n_top_genes)
-    return batch_scale(adata, chunk_size=chunk_size)
+        
+    if log: log.info('Batch specific maxabs scaling')
+    adata = batch_scale(adata, chunk_size=chunk_size)
+    
+    if log: log.info('Processed dataset shape: {}'.format(adata.shape))
+    return adata
     
 
 def batch_scale(adata, chunk_size=CHUNK_SIZE):
@@ -153,11 +185,10 @@ def batch_scale(adata, chunk_size=CHUNK_SIZE):
     """
     for b in adata.obs['batch'].unique():
         idx = np.where(adata.obs['batch']==b)[0]
-        scale_ = adata.X[idx].toarray().max(0)
-        adata.var['max_'+str(b)] = scale_
-        scale_ += (scale_ == 0)
-        for chunk, start, end in adata[idx].chunked_X(chunk_size):
-            adata.X[idx][start:end] = inplace_column_scale(chunk, scale_)
+        scaler = MaxAbsScaler(copy=False).fit(adata.X[idx])
+        for i in range(len(idx)//chunk_size+1):
+            adata.X[idx[i*chunk_size:(i+1)*chunk_size]] = scaler.transform(adata.X[idx[i*chunk_size:(i+1)*chunk_size]])
+
     return adata
         
 
@@ -230,17 +261,19 @@ def load_dataset(
         join='inner', 
         batch_key='batch', 
         batch_name='batch',
-         min_genes=600, 
+        min_genes=600, 
         min_cells=3, 
         n_top_genes=2000, 
         batch_size=64, 
-        chunk_size=CHUNK_SIZE
+        chunk_size=CHUNK_SIZE,
+        partial=False,
+        log=None,
     ):
     """
     Load dataset with preprocessing
     """
     adata = concat_data(data_list, batch_categories, join=join, batch_key=batch_key)
-    print('Raw dataset shape: {}'.format(adata.shape))
+    if log: log.info('Raw dataset shape: {}'.format(adata.shape))
     if batch_name!='batch':
         adata.obs['batch'] = adata.obs[batch_name]
     if 'batch' not in adata.obs:
@@ -253,8 +286,9 @@ def load_dataset(
         min_cells=min_cells, 
         n_top_genes=n_top_genes,
         chunk_size=chunk_size,
+        log=log,
     )
-    scdata = SingleCellDataset(adata)
+    scdata = SingleCellDataset(adata) # Wrap AnnData into Pytorch Dataset
     trainloader = DataLoader(
         scdata, 
         batch_size=batch_size, 
@@ -263,6 +297,9 @@ def load_dataset(
         num_workers=4
     )
     batch_sampler = BatchSampler(batch_size, adata.obs['batch'], drop_last=False)
-    testloader = DataLoader(scdata, batch_sampler=batch_sampler)
+    if partial:
+        testloader = DataLoader(scdata, batch_size=batch_size, drop_last=False, shuffle=True)
+    else:
+        testloader = DataLoader(scdata, batch_sampler=batch_sampler)
     
     return adata, trainloader, testloader 
\ No newline at end of file
