@@ -37,14 +37,16 @@ def SCALEX(
         seed=124, 
         gpu=0, 
         outdir='output/', 
-        projection=None, 
+        projection=None,
+        repeat=False,
         impute=None, 
-        chunk_size=100000,
+        chunk_size=20000,
         ignore_umap=False,
+        partial=False,
         verbose=False,
     ):
     
-    # sc.logging.print_versions()
+    sc.logging.print_versions()
     np.random.seed(seed) # seed
     torch.manual_seed(seed)
 
@@ -55,10 +57,9 @@ def SCALEX(
         device='cpu'
     
     outdir = outdir+'/'
+    os.makedirs(outdir+'/checkpoint', exist_ok=True)
+    log = create_logger('', fh=outdir+'log.txt')
     if not projection:
-        os.makedirs(outdir+'/checkpoint', exist_ok=True)
-        log = create_logger('', fh=outdir+'log.txt')
-        
         adata, trainloader, testloader = load_dataset(
             data_list, batch_categories, 
             join=join, 
@@ -68,9 +69,11 @@ def SCALEX(
             min_genes=min_genes, 
             min_cells=min_cells,
             batch_name=batch_name, 
-            batch_key=batch_key
+            batch_key=batch_key,
+            partial=partial,
+            log=log
         )
-        log.info('Processed dataset shape: {}'.format(adata.shape))
+#         log.info('Processed dataset shape: {}'.format(adata.shape))
         
         early_stopping = EarlyStopping(patience=10, checkpoint_file=outdir+'/checkpoint/model.pt')
         x_dim, n_domain = adata.shape[1], len(adata.obs['batch'].cat.categories)
@@ -88,9 +91,6 @@ def SCALEX(
         )
         torch.save({'n_top_genes':adata.var.index, 'enc':enc, 'dec':dec, 'n_domain':n_domain}, outdir+'/checkpoint/config.pt')     
     else:
-        os.makedirs(outdir, exist_ok=True)
-        log = create_logger('', fh=outdir+'log.txt')
-        
         state = torch.load(projection+'/checkpoint/config.pt')
         n_top_genes, enc, dec, n_domain = state['n_top_genes'], state['enc'], state['dec'], state['n_domain']
         model = VAE(enc, dec, n_domain=n_domain)
@@ -104,16 +104,18 @@ def SCALEX(
             n_top_genes=n_top_genes, 
             min_cells=0,
             batch_name=batch_name,
-            batch_key=batch_key
+            batch_key=batch_key,
+            partial=partial,
+            log = log
         )
-        log.info('Processed dataset shape: {}'.format(adata.shape))
+#         log.info('Processed dataset shape: {}'.format(adata.shape))
         
     adata.obsm['latent'] = model.encodeBatch(testloader, device=device) # save latent rep
     if impute:
         adata.layers['impute'] = model.encodeBatch(testloader, out='impute', device=device)
         
     log.info(outdir)
-    if projection:
+    if projection and (not repeat):
         ref = sc.read_h5ad(projection+'/adata.h5ad')
         adata = AnnData.concatenate(
             ref, adata, 
@@ -133,7 +135,7 @@ def SCALEX(
         cols = ['batch', 'celltype', 'leiden']
         color = [c for c in cols if c in adata.obs]
         if len(color) > 0:
-            if projection:
+            if projection and (not repeat):
                 plot_umap(adata, cond1='projection', save='.pdf')
             else:
                 sc.pl.umap(adata, color=color, save='.pdf', wspace=0.4, ncols=4)
